function [C,t,lowen] = compute_chroma(filename,thres)

% % % Cette fonction calcule le chromagramme  partir d'un fichier audio
% % % Input :
% % %     * filename : nom du fichier audio
% % %     * thres : seuil permettant de dtecter les silences
% % % Output :
% % %     * C : chromagramme
% % %     * t : vecteur temps
% % %     * lowen : indices des trames de silence
% % %     * audiodata1 : fichier audio mono
% % %     * Fs : frquence d'chantillonage du fichier audio


%% Paramtres initiaux

Fsdownsampled=11025/2; % Frquence d'chantillonnage souhaite
Nwindow=4096; % Longueur de fentre
Nhop=512; % Longueur de saut
if nargin<2
    thres=-55; % Valeur par dfaut du seuil
end


%% Lecture du fichier et rchantillonage


% Lecture du fichier son
[audiodata,Fs] = wavread(filename);


% On rend le signal monophonique
audiodata = mean(audiodata,2);

% On calcul le paramtre de rechantillonage:
downsample = round(Fs/Fsdownsampled);

% On rchantillonne
[audiodata,Fsdownsampled] = audiodownsample(audiodata,Fs,downsample);


% On fait du zero padding 
pad = zeros((Nwindow/2),1);
ilength = length(audiodata);
audiodata = [pad; audiodata; pad; pad];
    
% Calcul du nombre de trames
Ntrames = floor(ilength/Nhop) + 1;

% Calcul du vecteur temps
t = ((0 : Ntrames - 1).*Nhop)./(Fsdownsampled);
    
    
%% Calcul de la CQT

% Paramtres de la CQT
binsperoctave=36;
numcqbins=108;
fmin=73.42;
load('sparkernel.mat');

% Calcul de la STFT
winfft = hanning(Nwindow);
nfft=   size(sparkernel,1);
[fftframes]=stft(audiodata,nfft,winfft,Nhop,Ntrames);


% Calcul de la CQT
cq = (fftframes' * sparkernel)';

% Calcul du chromagramme et dplacement pour que le A440 soit le bin de
% rfrence
chroma=circshift(cq2hpcp(cq,Ntrames, numcqbins, binsperoctave),[0 ((binsperoctave/12)+1)/2 - 1]);


%% Algorithme de tuning

binshift=chromabias(chroma,binsperoctave); 
% Traitement du chromagramme pour prendre en compte le tuning
chroma=circshift(chroma',[-(binsperoctave/12 * (48-cps2midi(fmin))) - binshift 0]);

    
%% Post-processing     
    
% Filtrage median
chroma=medfilt1(chroma,8,[],2);
    
% Calcul des trames de silence
E = sum(chroma.^2,1);
lowen = find(10*log10(E+eps)<thres);
if ~isempty(lowen), pos = find(diff(lowen)>1);
    if size(pos)>1, lowen = [lowen(1:min(pos)) lowen(max(pos) + 1: end)]; end;
end

% Calcul du chromagramme sur 12 bins
tmp = reshape(chroma,binsperoctave/12,12,size(chroma,2));
tmp = tmp.*repmat(gausswin(3),[1 12 size(chroma,2)]);
chroma = reshape(sum(tmp,1),12,size(chroma,2));

% Normalisation du chromagramme
C=chroma/(alpha_norm(alpha_norm(chroma,6.63261),6.63261)+eps);

    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function binshift=chromabias(chroma,binsperoctave)
% finds bias on peak locations in the chromagram
% binshift is a corrective factor by which the chromagram
% must be circularly shifted to compensate for the bias
res = binsperoctave/12;

if res ~= 1 % this is to avoid shifting 12-dimensional chroma
    % adds last row at the top and first row at the bottom to create
    % circularity - effectively adds 2 to the bpo-length of the vectors:
    new = [chroma(end,:); chroma; chroma(1,:)];
    % pick peaks in the chroma
    pp=[];
    for k=1:size(new,1)
        pp=[pp peakpick(new(k,:))];
    end
    % modularises to res = bpo/12 bins:
    pp = mod(pp,res);
    % corrects the mod value for bin 3
    pp(pp==0)=res;
    % finds the highest concentration of peaks on the bpo/12 bin resolution
    [trash,pos]=max(hist(pp,res));
    % selects the shift
    binshift=pos-((res+1)/2);
else
    binshift = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [hpcpframes] = cq2hpcp(cqframes, Ntrames, numcqbins, binsperoctave)
% CQ2HPCP Convert Constant Q spectrum to a HPCP
%
%   CQ2HPCP(cqframes, info)
%
%   cqframes = vector or matrix constant q spectrum / spectra
%   info = info structure containing parameters
%

%get cq parameters from info structure
num_win = Ntrames;

cqbins = numcqbins;

bpo = binsperoctave;

% calculate the number of octaves we have in the spectrum
num_oct = floor(cqbins/bpo)-1;

hpcpframes(1:num_win, 1:bpo) = 0;



% for each window calculate chromagram
for i = 1 : num_win
    % add each octave of the spectrum into the chromagram
    for oct = 0:num_oct;
        ind = oct*bpo;

        hpcpframes(i, 1:bpo) = hpcpframes(i, 1:bpo) + abs(cqframes(ind+1:ind+bpo,i))';

    end;

    %     % normalise
    %     if max(hpcpframes(i,1:bpo)) > 0
    %         hpcpframes(i,1:bpo) = hpcpframes(i,1:bpo)/max(hpcpframes(i,1:bpo));
    %     else
    %         hpcpframes(i,1:bpo) = 0;
    %     end
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [xdown,newfs] = audiodownsample(xin,fs,fact)
% [xdown,newfs] = audiodownsample(xin,fs,fact)
% downsamples the signal (xin) by a factor (fact)
% fact must be an integer value
%
% by Juan, 05/05/05

if nargin<3, fact = 1; end;

% forces fact to be an integer
fact = floor(fact);
% normalize
xin = xin/(max(xin)+eps);

if fact <= 1
    xdown = xin;
    newfs = fs;
else
    % defines the filter specs,
    [b,a] = ellip(ellipord(0.9/fact, 1/fact, 0.001, 20),0.001,20,0.9/fact);
    % filters the signal
    xin_filt = filter(b,a,xin);
    %% downsample
    xdown = xin_filt(1:fact:end);
    newfs = fs/fact;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function T=peakpick(X)
% T=peakpick(X): return indices of elements of X which are peaks and positive

I=2:(length(X)-1);
T=find((X(I)>X(I-1)) & (X(I)>X(I+1)) & (X(I)>0))+1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function midi=cps2midi(cps)
midi=round(69+12*(log2(cps/440)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function l=alpha_norm(DF,alpha)
% l=alpha_norm(X,alpha): return alpha-norm norm.
% eg, alpha=2 means normalise
% variance. alpha=1 means normalise abs value. As alpha
% goes large, tends to normalisation by max value.
l=real(mean(abs(DF).^alpha).^(1/alpha));



function [S]=stft(x,nfft,w,Nhop,Ntrames)


% Traitement prliminaire : mettre les donnes en colonnes
x=x(:);
w=w(:);

% Parser le signal d'entre
Nwindow=length(w);

data=zeros(Nwindow,Ntrames);

for i=1:Ntrames
    data(:,i)=x(1+(i-1)*Nhop:Nwindow+(i-1)*Nhop);
end


% Fentrer
data=w(:,ones(1,Ntrames)).*data;

% Prendre la FFT 

S=conj(fft(data,nfft));












